total.est.fcn <- function(z, wgt, x=NULL, y=NULL, stratum=NULL, cluster=NULL,
   N.cluster=NULL, wgt1=NULL, x1=NULL, y1=NULL, popsize=NULL, stage1size=NULL,
   support=NULL, swgt=NULL, swgt1=NULL, unitsize=NULL, vartype="Local", conf=95,
   check.ind=TRUE) {


# Function:   total.est.fcn
# Programmer: Tom Kincaid
# Date:       February 19, 2004
# Description:
#   This function calculates estimates of the population total, mean, variance, 
#   and standard deviation of a response variable, where the response variable
#   may be defined for either a discrete or an extensive resource.  In addition 
#   the standard error of the population estimates and confidence bounds are 
#   calculated.  The Horvitz-Thompson estimator is used to calculate the 
#   total, variance, and standard deviation estimates.  The Horvitz-Thompson 
#   ratio estimator, i.e., the ratio of two Horvitz-Thompson estimators, is used 
#   to calculate the mean estimate.  Variance estimates are calculated using 
#   either the local mean variance estimator or the simple random sampling (SRS) 
#   variance estimator.  The choice of variance estimator is subject to user 
#   control.  The local mean variance estimator requires the x-coordinate and the
#   y-coordinate of each site.  The SRS variance estimator uses the independent 
#   random sample approximation to calculate joint inclusion probabilities.  
#   Confidence bounds are calculated using a Normal distribution multiplier.
#   The function can accommodate a stratified sample.  For a stratified sample, 
#   separate estimates and standard errors are calculated for each stratum, which
#   are used to produce estimates and standard errors for all strata combined.  
#   Strata that contain a single value are removed.  For a stratified sample, 
#   when either the size of the resource or the sum of the size-weights of the 
#   resource is provided for each stratum, those values are used as stratum 
#   weights for calculating the estimates and standard errors for all strata 
#   combined.  For a stratified sample when neither the size of the resource nor 
#   the sum of the size-weights of the resource is provided for each stratum, 
#   estimated values are used as stratum weights for calculating the estimates 
#   and standard errors for all strata combined.  The function can accomodate 
#   single-stage and two-stage samples for both stratified and unstratified 
#   sampling designs.  Finite population and continuous population correction 
#   factors can be utilized in variance estimation.  The function checks for 
#   compatibility of input values and removes missing values.
#   Input:
#      z = the response value for each site.
#      wgt = the final adjusted weight (inverse of the sample inclusion
#         probability) for each site, which is either the weight for a single-
#         stage sample or the stage two weight for a two-stage sample.
#      x = x-coordinate for location for each site, which is either the x-
#          coordinate for a single-stage sample or the stage two x-coordinate
#          for a two-stage sample.  The default is NULL.
#      y = y-coordinate for location for each site, which is either the y-
#          coordinate for a single-stage sample or the stage two y-coordinate
#          for a two-stage sample.  The default is NULL.
#      stratum = the stratum for each site.  The default is NULL.
#      cluster = the stage one sampling unit (primary sampling unit or cluster) 
#         code for each site.  The default is NULL.
#      N.cluster = the number of stage one sampling units in the resource, which 
#         is required for calculation of finite and continuous population 
#         correction factors for a two-stage sample.  For a stratified sample 
#         this variable must be a vector containing a value for each stratum and
#         must have the names attribute set to identify the stratum codes.  The
#         default is NULL.
#      x1 = the stage one x-coordinate for location for each site.  The default
#         is NULL.
#      x1 = the stage one x-coordinate for location for each site.  The default
#         is NULL.
#      y1 = the stage one y-coordinate for location for each site.  The default
#         is NULL.
#      popsize = the known size of the resource - the total number of sampling 
#         units of a discrete resource or the measure of a continuous resource,
#         which is required for calculation of finite and continuous population 
#         correction factors for a single-stage sample.  This variable is also 
#         used to adjust estimators for the known size of a resource.  For a
#         stratified sample this variable must be a vector containing a value 
#         for each stratum and must have the names attribute set to identify the
#         stratum codes.  The default is NULL.
#      stage1size = the known size of the stage one sampling units of a two-
#         stage sample, which is required for calculation of finite and  
#         continuous population correction factors for a two-stage sample and 
#         must have the names attribute set to identify the stage one sampling 
#         unit codes.  For a stratified sample, the names attribute must be set
#         to identify both stratum codes and stage one sampling unit codes using
#         a convention where the two codes are separated by the # symbol, e.g.,
#         "Stratum 1#Cluster 1".  The default is NULL.
#      support = the support value for each site - the value one (1) for a 
#         site from a discrete resource or the measure of the sampling unit  
#         associated with a site from a continuous resource, which is required  
#         for calculation of finite and continuous population correction  
#         factors.  The default is NULL.
#      swgt = the size-weight for each site, which is the stage two size-weight 
#         for a two-stage sample.  The default is NULL.
#      swgt1 = the stage one size-weight for each site.  The default is NULL.
#      unitsize = the known sum of the size-weights of the resource, which for a 
#         stratified sample must be a vector containing a value for each stratum 
#         and must have the names attribute set to identify the stratum codes.  
#         The default is NULL.
#      vartype = the choice of variance estimator, where "Local" = local mean
#         estimator and "SRS" = SRS estimator.  The default is "Local".
#      conf = the confidence level.  The default is 95%.
#      check.ind = a logical value that indicates whether compatability
#         checking of the input values is conducted, where TRUE = conduct 
#         compatibility checking and FALSE = do not conduct compatibility 
#         checking.  The default is TRUE.
#   Output:
#      An object in data frame format containing the estimates and confidence
#      bounds.
#   Other Functions Required:
#      input.check.fcn - check input values for compatibility
#      wnas.fcn - remove missing values
#      total.var.fcn - calculate variance of the total, mean, variance, and 
#         standard deviation estimates
#   Examples:
#      z <- rnorm(100, 10, 1)
#      wgt <- runif(100, 10, 100)
#      total.est.fcn(z, wgt, vartype="SRS")
#
#      x <- runif(100)
#      y <- runif(100)
#      total.est.fcn(z, wgt, x, y)


# Attach required libraries

   if(vartype == "Local") {
      if(software.program == "R") {
         library(MASS)
      } else if(software.program == "S-PLUS") {
         library(Mass)
      }
   }

# Check for existence of the response variable and determine the number of values

   if(is.null(z))
      stop("\nValues for the response variable must be provided.")
   if(!is.numeric(z))
      stop("\nValues for the response variable must be numeric.")
   nresp <- length(z)

# Determine whether the sample is stratified

   stratum.ind <- length(unique(stratum)) > 1

# If the sample is stratified, convert stratum to a factor, determine stratum 
# levels, and calculate number of strata,

   if(stratum.ind) {
      stratum <- factor(stratum)
      stratum.levels <- levels(stratum)
      nstrata <- length(stratum.levels)
   } else {
      stratum.levels <- NULL
      nstrata <- NULL
   }

# Determine whether the sample has two stages

   cluster.ind <- length(unique(cluster)) > 1

# Determine whether the population correction factor is to be used

   pcfactor.ind <- !is.null(support)

# Determine whether the sample uses size-weights

   swgt.ind <- length(swgt) > 0

# Begin the section that checks for compatibility of input values

   if(check.ind) {

# If the sample has two stages, convert cluster to a factor, determine cluster 
# levels, and calculate number of clusters

   if(cluster.ind) {
      if(stratum.ind) {
         cluster.in <- cluster
         cluster <- tapply(cluster, stratum, factor)
         cluster.levels <- sapply(cluster, levels, simplify=FALSE)
         ncluster <- sapply(cluster.levels, length)
      } else {
         cluster <- factor(cluster)
         cluster.levels <- levels(cluster)
         ncluster <- length(cluster.levels)
      }
   }

# Check for compatibility of input values

      temp <- input.check.fcn(nresp, wgt, NULL, NULL, x, y, stratum.ind, stratum, stratum.levels, nstrata, cluster.ind, cluster, cluster.levels, ncluster, N.cluster, wgt1, x1, y1, popsize, stage1size, pcfactor.ind, support, swgt.ind, swgt, swgt1, unitsize, vartype, conf)

      N.cluster <- temp$N.cluster
      popsize <- temp$popsize
      stage1size <- temp$stage1size
      unitsize <- temp$unitsize

# If the sample was stratified and had two stages, then reset cluster to its 
# input value

   if(stratum.ind && cluster.ind)
      cluster <- cluster.in

# End the section that checks for compatibility of input values

   }

# Remove missing values

   if(vartype == "Local") {
      if(swgt.ind) {
         if(stratum.ind) {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y, stratum=stratum, cluster=cluster, wgt1=wgt1, x1=x1, y1=y1, swgt=swgt, swgt1=swgt1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y, stratum=stratum, swgt=swgt))
         } else {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y, cluster=cluster, wgt1=wgt1, x1=x1, y1=y1, swgt=swgt, swgt1=swgt1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y, swgt=swgt))
         }
         z <- temp$z
         wgt <- temp$wgt
         x <- temp$x
         y <- temp$y
         if(stratum.ind)
            stratum <- temp$stratum
         if(cluster.ind) {
            cluster <- temp$cluster
            wgt1 <- temp$wgt1
            x1 <- temp$x1
            y1 <- temp$y1
            swgt1 <- temp$swgt1
         }
         swgt <- temp$swgt
      } else {
         if(stratum.ind) {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y, stratum=stratum, cluster=cluster, wgt1=wgt1, x1=x1, y1=y1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y, stratum=stratum))
         } else {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y, cluster=cluster, wgt1=wgt1, x1=x1, y1=y1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt, x=x, y=y))
         }
         z <- temp$z
         wgt <- temp$wgt
         x <- temp$x
         y <- temp$y
         if(stratum.ind)
            stratum <- temp$stratum
         if(cluster.ind) {
            cluster <- temp$cluster
            wgt1 <- temp$wgt1
            x1 <- temp$x1
            y1 <- temp$y1
         }
      }
   } else {
      if(swgt.ind) {
         if(stratum.ind) {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, stratum=stratum, cluster=cluster, wgt1=wgt1, swgt=swgt, swgt1=swgt1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt, stratum=stratum, swgt=swgt))
         } else {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, cluster=cluster, wgt1=wgt1, swgt=swgt, swgt1=swgt1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt, swgt=swgt))
         }
         z <- temp$z
         wgt <- temp$wgt
         if(stratum.ind)
            stratum <- temp$stratum
         if(cluster.ind) {
            cluster <- temp$cluster
            wgt1 <- temp$wgt1
            swgt1 <- temp$swgt1
         }
         swgt <- temp$swgt
      } else {
         if(stratum.ind) {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, stratum=stratum, cluster=cluster, wgt1=wgt1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt, stratum=stratum))
         } else {
            if(cluster.ind)
               temp <- wnas.fcn(list(z=z, wgt=wgt, cluster=cluster, wgt1=wgt1))
            else
               temp <- wnas.fcn(list(z=z, wgt=wgt))
         }
         z <- temp$z
         wgt <- temp$wgt
         if(stratum.ind)
            stratum <- temp$stratum
         if(cluster.ind) {
            cluster <- temp$cluster
            wgt1 <- temp$wgt1
         }
      }
   }

# For a stratified sample, remove strata that have a single value

   if(stratum.ind) {
      stratum <- factor(stratum)
      stratum.levels <- levels(stratum)
      nstrata <- length(stratum.levels)
      ind <- FALSE
      for(i in 1:nstrata) {
         stratum.i <- stratum == stratum.levels[i]
         if(sum(stratum.i) == 1) {
            warning(paste("The stratum named", stratum.levels[i], "contains a single value and was removed.\n"))
            z <- z[!stratum.i]
            wgt <- wgt[!stratum.i]
            if(vartype == "Local") {
               x <- x[!stratum.i]
               y <- y[!stratum.i]
            }
            stratum <- stratum[!stratum.i]
            if(cluster.ind) {
               cluster <- cluster[!stratum.i]
               wgt1 <- wgt1[!stratum.i]
               if(vartype == "Local") {
                  x1 <- x1[!stratum.i]
                  y1 <- y1[!stratum.i]
               }
            }
            if(swgt.ind) {
               swgt <- swgt[!stratum.i]
               if(cluster.ind)
                  swgt1 <- swgt1[!stratum.i]
            }
            if(length(popsize) > 0)
               popsize <- popsize[names(popsize) != stratum.levels[i]]
            if(length(unitsize) > 0)
               unitsize <- unitsize[names(unitsize) != stratum.levels[i]]
            ind <- TRUE
         }
      }
      if(ind) {
         stratum <- factor(stratum)
         stratum.levels <- levels(stratum)
         nstrata <- length(stratum.levels)

# Check whether the number of strata is one

         if(nstrata == 1)
            warning("Only a single stratum is available for the analysis.\n")
     }
   }

# Check whether the vector of response values is empty

   if(length(z) == 0)
      stop("\nEstimates cannot be calculated since the vector of response values is empty.")

# Calculate confidence bound multiplier

   mult <- qnorm(0.5 + (conf/100)/2)

# Calculate additional required values

   if(swgt.ind) {
      if(length(unitsize) > 0)
         sum.unitsize <- sum(unitsize)
      if(stratum.ind) {
         if(cluster.ind) {
            unitsize.hat <- tapply(wgt*swgt*wgt1*swgt1, stratum, sum)
            sum.unitsize.hat <- sum(wgt*swgt*wgt1*swgt1)
         } else {
            unitsize.hat <- tapply(wgt*swgt, stratum, sum)
            sum.unitsize.hat <- sum(wgt*swgt)
         }
      } else {
         if(cluster.ind)
            unitsize.hat <- sum(wgt*swgt*wgt1*swgt1)
         else
            unitsize.hat <- sum(wgt*swgt)
      }
   } else {
      if(length(popsize) > 0)
         sum.popsize <- sum(popsize)
      if(stratum.ind) {
         if(cluster.ind) {
            popsize.hat <- tapply(wgt*wgt1, stratum, sum)
            sum.popsize.hat <- sum(wgt*wgt1)
         } else {
            popsize.hat <- tapply(wgt, stratum, sum)
            sum.popsize.hat <- sum(wgt)
         }
      } else {
         if(cluster.ind)
            popsize.hat <- sum(wgt*wgt1)
         else
            popsize.hat <- sum(wgt)
      }
   }

# Branch to handle stratified and unstratified data

   if(stratum.ind) {

# Begin the section for stratified data

# Create the data frame for estimates for all strata combined

   Results <- data.frame(array(0, c(4, 6)))
   dimnames(Results) <- list(1:4, c("Statistic", "NResp", "Estimate.U", "StdError.U", paste("LCB", conf, "Pct.U", sep=""), paste("UCB", conf, "Pct.U", sep="")))
   Results[,1] <- c("Total", "Mean", "Variance", "Std. Deviation")
   Results[,2] <- rep(length(z), 4)

# Begin the subsection for individual strata

   for(i in 1:nstrata) {

# Calculate required values

      stratum.i <- stratum == stratum.levels[i]
      z.st <- z[stratum.i]
      if(swgt.ind) {
         if(cluster.ind) {
            w2 <- wgt[stratum.i]*swgt[stratum.i]
            w1 <- wgt1[stratum.i]*swgt1[stratum.i]
         } else {
            w <- wgt[stratum.i]*swgt[stratum.i]
         }
      } else {
         if(cluster.ind) {
            w2 <- wgt[stratum.i]
            w1 <- wgt1[stratum.i]
         } else {
            w <- wgt[stratum.i]
         }
      }

# Calculate the total estimate

      if(cluster.ind)
         total.est <- sum(w2*w1*z.st)
      else
         total.est <- sum(w*z.st)

# Calculate the mean estimate

      if(swgt.ind)
         mean.est <- total.est / unitsize.hat[i]
      else
         mean.est <- total.est / popsize.hat[i]

# Calculate the variance and standard deviation estimates

      if(swgt.ind) {
         if(cluster.ind)
            var.est <- sum(w2*w1*((z.st - mean.est)^2)) / (unitsize.hat[i] - 1)
          else
            var.est <- sum(w*((z.st - mean.est)^2)) / (unitsize.hat[i] - 1)
      } else {
         if(cluster.ind)
            var.est <- sum(w2*w1*((z.st - mean.est)^2)) / (popsize.hat[i] - 1)
          else
            var.est <- sum(w*((z.st - mean.est)^2)) / (popsize.hat[i] - 1)
      }
      sd.est <- sqrt(var.est)

# Calculate variance estimates for the estimated quantities

      if(cluster.ind)
         temp <- total.var.fcn(z.st, w2, x[stratum.i], y[stratum.i], mean.est, var.est, sd.est, stratum.ind, stratum.levels[i], cluster.ind, cluster[stratum.i], N.cluster[i], w1, x1[stratum.i], y1[stratum.i], popsize[i], pcfactor.ind, stage1size[[i]], support[stratum.i], vartype)
      else
         temp <- total.var.fcn(z.st, w, x[stratum.i], y[stratum.i], mean.est, var.est, sd.est, stratum.ind, stratum.levels[i], cluster.ind, popsize=popsize[i], pcfactor.ind=pcfactor.ind, support=support[stratum.i], vartype=vartype)
      total.var <- temp[1]
      mean.var <- temp[2]
      var.var <- temp[3]
      sd.var <- temp[4]

# Add estimates to the data frame for all strata combined

      Results[1, 3] <- Results[1, 3] + total.est
      Results[1, 4] <- Results[1, 4] + total.var
      if(swgt.ind) {
         if(length(unitsize) > 0) {
            Results[2:4, 3] <- Results[2:4, 3] + (unitsize[i]/sum.unitsize)*c(mean.est, var.est, sd.est)
            Results[2:4, 4] <- Results[2:4, 4] + ((unitsize[i]/sum.unitsize)^2)*c(mean.var, var.var, sd.var)
         } else {
            Results[2:4, 3] <- Results[2:4, 3] + (unitsize.hat[i]/sum.unitsize.hat)*c(mean.est, var.est, sd.est)
            Results[2:4, 4] <- Results[2:4, 4] + ((unitsize.hat[i]/sum.unitsize.hat)^2)*c(mean.var, var.var, sd.var)
         }
      } else {
         if(length(popsize) > 0) {
            Results[2:4, 3] <- Results[2:4, 3] + (popsize[i]/sum.popsize)*c(mean.est, var.est, sd.est)
            Results[2:4, 4] <- Results[2:4, 4] + ((popsize[i]/sum.popsize)^2)*c(mean.var, var.var, sd.var)
         } else {
            Results[2:4, 3] <- Results[2:4, 3] + (popsize.hat[i]/sum.popsize.hat)*c(mean.est, var.est, sd.est)
            Results[2:4, 4] <- Results[2:4, 4] + ((popsize.hat[i]/sum.popsize.hat)^2)*c(mean.var, var.var, sd.var)
         }
      }

# End the subsection for individual strata

   }

# Begin the subsection for all strata combined

# Calculate standard errors and confidence bounds

   Results[, 4] <- sqrt(Results[, 4])
   Results[, 5] <- Results[, 3] - mult*Results[, 4]
   Results[3:4, 5] <- pmax(Results[3:4, 5], 0)
   Results[, 6] <- Results[, 3] + mult*Results[, 4]

# End the subsection for all strata combined

# End the section for stratified data

   } else {

# Begin the section for unstratified data

# Check whether the vector of response values contains a single element

   if(length(z) == 1)
      stop("\nEstimates cannot be calculated since the vector of response values contains a \nsingle element.")

# Create the data frame for estimates

   Results <- data.frame(array(0, c(4, 6)))
   dimnames(Results) <- list(1:4, c("Statistic", "NResp", "Estimate.U", "StdError.U", paste("LCB", conf, "Pct.U", sep=""), paste("UCB", conf, "Pct.U", sep="")))
   Results[,1] <- c("Total", "Mean", "Variance", "Std. Deviation")
   Results[,2] <- rep(length(z), 4)

# Calculate required values

   if(swgt.ind) {
      if(cluster.ind) {
         w2 <- wgt*swgt
         w1 <- wgt1*swgt1
      } else {
         w <- wgt*swgt
      }
   } else {
      if(cluster.ind) {
         w2 <- wgt
         w1 <- wgt1
      } else {
         w <- wgt
      }
   }

# Calculate the total estimate

   if(cluster.ind)
      total.est <- sum(w2*w1*z)
   else
      total.est <- sum(w*z)

# Calculate the mean estimate

   if(swgt.ind)
      mean.est <- total.est / unitsize.hat
   else
      mean.est <- total.est / popsize.hat

# Calculate the variance and standard deviation estimates

   if(swgt.ind) {
      if(cluster.ind)
         var.est <- sum(w2*w1*((z - mean.est)^2)) / (unitsize.hat - 1)
       else
         var.est <- sum(w*((z - mean.est)^2)) / (unitsize.hat - 1)
   } else {
      if(cluster.ind)
         var.est <- sum(w2*w1*((z - mean.est)^2)) / (popsize.hat - 1)
       else
         var.est <- sum(w*((z - mean.est)^2)) / (popsize.hat - 1)
   }
   sd.est <- sqrt(var.est)

# Calculate standard error estimates for the estimated quantities

      if(cluster.ind)
         temp <- total.var.fcn(z, w2, x, y, mean.est, var.est, sd.est, stratum.ind, NULL, cluster.ind, cluster, N.cluster, w1, x1, y1, popsize, pcfactor.ind, stage1size, support, vartype)
      else
         temp <- total.var.fcn(z, w, x, y, mean.est, var.est, sd.est, stratum.ind,  NULL, cluster.ind, popsize=popsize, pcfactor.ind=pcfactor.ind, support=support, vartype=vartype)
      total.sd <- sqrt(temp[1])
      mean.sd <- sqrt(temp[2])
      var.sd <- sqrt(temp[3])
      sd.sd <- sqrt(temp[4])

# Add estimates to the data frame for results

   Results[, 3] <- c(total.est, mean.est, var.est, sd.est)
   Results[, 4] <- c(total.sd, mean.sd, var.sd, sd.sd)

# Calculate confidence bounds

   Results[, 5] <- Results[, 3] - mult*Results[, 4]
   Results[3:4, 5] <- pmax(Results[3:4, 5], 0)
   Results[, 6] <- Results[, 3] + mult*Results[, 4]

# End the section for unstratified data

   }

# Return the data frame

   Results
}
